

class NiprovOptions(object):
    """General settings for niprov.

    Attributes:
        dryrun (bool): Do not execute commands or make lasting changes to the 
            provenance database. Defaults to False.
        verbose (bool): Output extra information. Defaults to False.
    """
    dryrun = False
    verbose = False
